/**
 * Copyright: Copyright (c) 2009 Jacob Carlborg.
 * Authors: Jacob Carlborg
 * Version: Initial created: Aug 3, 2009 
 * License: $(LINK2 http://www.boost.org/LICENSE_1_0.txt, Boost Software License 1.0)
 */
module dstep.foundation.NSError;

import dstep.foundation.NSArray;
import dstep.foundation.NSCoder;
import dstep.foundation.NSDictionary;
import dstep.foundation.NSObjCRuntime;
import dstep.foundation.NSObject;
import dstep.foundation.NSString;
import dstep.foundation.NSZone;
import dstep.objc.bridge.Bridge;
import dstep.objc.objc;



import bindings = dstep.foundation.NSError_bindings;

private
{
	NSString NSCocoaErrorDomain_;
	NSString NSPOSIXErrorDomain_;
	NSString NSOSStatusErrorDomain_;
	NSString NSMachErrorDomain_;
	NSString NSUnderlyingErrorKey_;
	NSString NSLocalizedDescriptionKey_;
	NSString NSLocalizedFailureReasonErrorKey_;
	NSString NSLocalizedRecoverySuggestionErrorKey_;
	NSString NSLocalizedRecoveryOptionsErrorKey_;
	NSString NSRecoveryAttempterErrorKey_;
	NSString NSStringEncodingErrorKey_;
	NSString NSURLErrorKey_;
	NSString NSFilePathErrorKey_;
}

NSString NSCocoaErrorDomain ()
{
	if (NSCocoaErrorDomain_)
		return NSCocoaErrorDomain_;
		
	return NSCocoaErrorDomain_ = new NSString(bindings.NSCocoaErrorDomain);
}

NSString NSPOSIXErrorDomain ()
{
	if (NSPOSIXErrorDomain_)
		return NSPOSIXErrorDomain_;
		
	return NSPOSIXErrorDomain_ = new NSString(bindings.NSPOSIXErrorDomain);
}

NSString NSOSStatusErrorDomain ()
{
	if (NSOSStatusErrorDomain_)
		return NSOSStatusErrorDomain_;
		
	return NSOSStatusErrorDomain_ = new NSString(bindings.NSOSStatusErrorDomain);
}

NSString NSMachErrorDomain ()
{
	if (NSMachErrorDomain_)
		return NSMachErrorDomain_;
		
	return NSMachErrorDomain_ = new NSString(bindings.NSMachErrorDomain);
}

NSString NSUnderlyingErrorKey ()
{
	if (NSUnderlyingErrorKey_)
		return NSUnderlyingErrorKey_;
		
	return NSUnderlyingErrorKey_ = new NSString(bindings.NSUnderlyingErrorKey);
}

NSString NSLocalizedDescriptionKey ()
{
	if (NSLocalizedDescriptionKey_)
		return NSLocalizedDescriptionKey_;
		
	return NSLocalizedDescriptionKey_ = new NSString(bindings.NSLocalizedDescriptionKey);
}

NSString NSLocalizedFailureReasonErrorKey ()
{
	if (NSLocalizedFailureReasonErrorKey_)
		return NSLocalizedFailureReasonErrorKey_;
		
	return NSLocalizedFailureReasonErrorKey_ = new NSString(bindings.NSLocalizedFailureReasonErrorKey);
}

NSString NSLocalizedRecoverySuggestionErrorKey ()
{
	if (NSLocalizedRecoverySuggestionErrorKey_)
		return NSLocalizedRecoverySuggestionErrorKey_;
		
	return NSLocalizedRecoverySuggestionErrorKey_ = new NSString(bindings.NSLocalizedRecoverySuggestionErrorKey);
}

NSString NSLocalizedRecoveryOptionsErrorKey ()
{
	if (NSLocalizedRecoveryOptionsErrorKey_)
		return NSLocalizedRecoveryOptionsErrorKey_;
		
	return NSLocalizedRecoveryOptionsErrorKey_ = new NSString(bindings.NSLocalizedRecoveryOptionsErrorKey);
}

NSString NSRecoveryAttempterErrorKey ()
{
	if (NSRecoveryAttempterErrorKey_)
		return NSRecoveryAttempterErrorKey_;
		
	return NSRecoveryAttempterErrorKey_ = new NSString(bindings.NSRecoveryAttempterErrorKey);
}

NSString NSStringEncodingErrorKey ()
{
	if (NSStringEncodingErrorKey_)
		return NSStringEncodingErrorKey_;
		
	return NSStringEncodingErrorKey_ = new NSString(bindings.NSStringEncodingErrorKey);
}

NSString NSURLErrorKey ()
{
	if (NSURLErrorKey_)
		return NSURLErrorKey_;
		
	return NSURLErrorKey_ = new NSString(bindings.NSURLErrorKey);
}

NSString NSFilePathErrorKey ()
{
	if (NSFilePathErrorKey_)
		return NSFilePathErrorKey_;
		
	return NSFilePathErrorKey_ = new NSString(bindings.NSFilePathErrorKey);
}

const TNSErrorRecoveryAttempting = `

	void attemptRecoveryFromError (NSError error, NSUInteger recoveryOptionIndex, Object delegate_, SEL didRecoverSelector, void* contextInfo)
	{
		return invokeObjcSelf!(void, "attemptRecoveryFromError:optionIndexdelegate:didRecoverSelector:contextInfo:", NSError, NSUInteger, Object, SEL, void*)(error, recoveryOptionIndex, delegate_, didRecoverSelector, contextInfo);
	}
	
	bool attemptRecoveryFromError (NSError error, NSUInteger recoveryOptionIndex)
	{
		return invokeObjcSelf!(bool, "attemptRecoveryFromError:optionIndex:", NSError, NSUInteger)(error, recoveryOptionIndex);
	}
	
	//mixin ObjcBindMethod!(attemptRecoveryFromError, void, "attemptRecoveryFromError:optionIndexdelegate:didRecoverSelector:contextInfo:", NSError, NSUInteger, Object, SEL, void*);
	//mixin ObjcBindMethod!(attemptRecoveryFromError, bool, "attemptRecoveryFromError:optionIndex:", NSError, NSUInteger);
`;

class NSError : NSObject, INSCopying, INSCoding
{
	mixin (ObjcWrap);
	
	this ()
	{
		super(typeof(this).alloc.init.objcObject);
	}
	
	typeof(this) init ()
	{
		return invokeObjcSelf!(typeof(this), "init");
	}

	Object initWithDomain (NSString domain, NSInteger code, NSDictionary dict)
	{
		return invokeObjcSelf!(Object, "initWithDomain:code:userInfo:", NSString, NSInteger, NSDictionary)(domain, code, dict);
	}

	this (NSString domain, NSInteger code, NSDictionary dict)
	{
		typeof(this).alloc.initWithDomain(domain, code, dict);
	}

	static Object errorWithDomain (NSString domain, NSInteger code, NSDictionary dict)
	{
		return invokeObjcSuperClass!(Object, "errorWithDomain:code:userInfo:", NSString, NSInteger, NSDictionary)(domain, code, dict);
	}

	NSString domain ()
	{
		return invokeObjcSelf!(NSString, "domain");
	}

	NSInteger code ()
	{
		return invokeObjcSelf!(NSInteger, "code");
	}

	NSDictionary userInfo ()
	{
		return invokeObjcSelf!(NSDictionary, "userInfo");
	}

	NSString localizedDescription ()
	{
		return invokeObjcSelf!(NSString, "localizedDescription");
	}

	NSString localizedFailureReason ()
	{
		return invokeObjcSelf!(NSString, "localizedFailureReason");
	}

	NSString localizedRecoverySuggestion ()
	{
		return invokeObjcSelf!(NSString, "localizedRecoverySuggestion");
	}

	NSArray localizedRecoveryOptions ()
	{
		return invokeObjcSelf!(NSArray, "localizedRecoveryOptions");
	}

	Object recoveryAttempter ()
	{
		return invokeObjcSelf!(Object, "recoveryAttempter");
	}

	Object copyWithZone (NSZone* zone)
	{
		return invokeObjcSelf!(Object, "copyWithZone:", NSZone*)(zone);
	}

	void encodeWithCoder (NSCoder aCoder)
	{
		return invokeObjcSelf!(void, "encodeWithCoder:", NSCoder)(aCoder);
	}

	Object initWithCoder (NSCoder aDecoder)
	{
		return invokeObjcSelf!(Object, "initWithCoder:", NSCoder)(aDecoder);
	}

	this (NSCoder aDecoder)
	{
		typeof(this).alloc.initWithCoder(aDecoder);
	}
}